﻿//
// DirectXPage.xaml.cpp
// Implementation of the DirectXPage.xaml class.
//

#include "pch.h"
#include "DirectXPage.xaml.h"

using namespace GeometryExperimentation;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::UI::Input;
using namespace Windows::UI::Core;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

DirectXPage::DirectXPage() : m_renderNeeded(true)
{
	InitializeComponent();

	m_renderer = ref new GeometryVarietiesRenderer();

	m_renderer->Initialize(
		Window::Current->CoreWindow,
		SwapChainPanel,
		DisplayProperties::LogicalDpi
		);

	Window::Current->CoreWindow->SizeChanged += 
		ref new TypedEventHandler<CoreWindow^, WindowSizeChangedEventArgs^>(this, &DirectXPage::OnWindowSizeChanged);

	DisplayProperties::LogicalDpiChanged +=
		ref new DisplayPropertiesEventHandler(this, &DirectXPage::OnLogicalDpiChanged);

	DisplayProperties::OrientationChanged +=
        ref new DisplayPropertiesEventHandler(this, &DirectXPage::OnOrientationChanged);

	DisplayProperties::DisplayContentsInvalidated +=
		ref new DisplayPropertiesEventHandler(this, &DirectXPage::OnDisplayContentsInvalidated);
	
	m_eventToken = CompositionTarget::Rendering::add(ref new EventHandler<Object^>(this, &DirectXPage::OnRendering));

	m_timer = ref new BasicTimer();

    Loaded += ref new RoutedEventHandler([this] (Object^, Object^)
    {
        // Check the first two radio buttons in each group
		for (UIElement^ item : fillModeStackPanel->Children)
		{
			auto radioButton = dynamic_cast<RadioButton^>(item);
			if (radioButton != nullptr)
			{
				radioButton->IsChecked = true;
				break;
			}
		}
		for (UIElement^ item : optionStackPanel->Children)
		{
			auto radioButton = dynamic_cast<RadioButton^>(item);
			if (radioButton != nullptr)
			{
				radioButton->IsChecked = true;
				break;
			}
		}
    });
}

void DirectXPage::OnWindowSizeChanged(CoreWindow^ sender, WindowSizeChangedEventArgs^ args)
{
	m_renderer->UpdateForWindowSizeChange();
	m_renderNeeded = true;
}

void DirectXPage::OnLogicalDpiChanged(Object^ sender)
{
	m_renderer->SetDpi(DisplayProperties::LogicalDpi);
	m_renderNeeded = true;
}

void DirectXPage::OnOrientationChanged(Object^ sender)
{
	m_renderer->UpdateForWindowSizeChange();
	m_renderNeeded = true;
}

void DirectXPage::OnDisplayContentsInvalidated(Object^ sender)
{
	m_renderer->ValidateDevice();
	m_renderNeeded = true;
}

void DirectXPage::OnRendering(Object^ sender, Object^ args)
{
	if (m_renderNeeded)
	{
		m_timer->Update();
		m_renderer->Update(m_timer->Total, m_timer->Delta);
		m_renderer->Render();
		m_renderer->Present();

        if (m_renderingOption != RenderingOption::AnimatedDotOffset)
            m_renderNeeded = false;
	}
}

void DirectXPage::SaveInternalState(IPropertySet^ state)
{
}

void DirectXPage::LoadInternalState(IPropertySet^ state)
{
}

void DirectXPage::OnRenderingOptionRadioButtonChecked(Object^ sender, RoutedEventArgs^ args)
{
    RenderingOptionRadioButton^ btn = safe_cast<RenderingOptionRadioButton^>(sender);
    m_renderingOption = btn->OptionTag;
    m_renderer->SetRenderingOption(m_renderingOption);
    m_renderNeeded = true;
}

void DirectXPage::OnFillModeRadioButtonChecked(Object^ sender, RoutedEventArgs^ args)
{
    RadioButton^ btn = safe_cast<RadioButton^>(sender);
    int fillMode = (int)btn->Tag;
    m_renderer->SetFillMode(fillMode);
    m_renderNeeded = true;
}
