﻿#pragma once

#include "DirectXBase.h"
#include "RenderingOption.h"

ref class GeometryVarietiesRenderer sealed : public DirectXBase
{
private:
    D2D1_FILL_MODE m_fillMode;
    GeometryExperimentation::RenderingOption m_renderingOption;

    Microsoft::WRL::ComPtr<ID2D1TransformedGeometry> m_starGeometry;
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_squareWaveGeometry;
    Microsoft::WRL::ComPtr<ID2D1TransformedGeometry> m_infinityGeometry;
    Microsoft::WRL::ComPtr<ID2D1GeometryGroup> m_geometryGroup;

    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_simplifiedGeometry;
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_grosslySimplifiedGeometry;
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_outlinedGeometry;
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_widenedGeometry;
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_outlinedWidenedGeometry;

    Microsoft::WRL::ComPtr<ID2D1StrokeStyle> m_roundedStrokeStyle;
    Microsoft::WRL::ComPtr<ID2D1StrokeStyle> m_dottedStrokeStyle;
    Microsoft::WRL::ComPtr<ID2D1StrokeStyle> m_animatedStrokeStyle;

	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> m_blackBrush;
	Microsoft::WRL::ComPtr<ID2D1SolidColorBrush> m_redBrush;

public:
    GeometryVarietiesRenderer();

	// DirectXBase methods.
	virtual void CreateDeviceIndependentResources() override;
	virtual void CreateDeviceResources() override;
	virtual void CreateWindowSizeDependentResources() override;
	virtual void Render() override;

	// Method for updating time-dependent objects.
	void Update(float timeTotal, float timeDelta);

    // Called from DirectXPage to set rendering option & fill mode
    void SetRenderingOption(GeometryExperimentation::RenderingOption renderingOption);
    void SetFillMode(int fillMode);

private:
    HRESULT CreateInfinitySign(float radius, ID2D1PathGeometry** ppPathGeometry);
    HRESULT CreateFivePointedStar(float radius, ID2D1PathGeometry** ppPathGeometry);
    void CreateGeometryGroup();
};
